﻿
rollout BackburnerSim "Phoenix FD Backburner Simulation"
(
	edittext managerText "Backburner Manager Name or IP:" fieldWidth:500 labelOnTop:true

	edittext groupText "Backburner Server Group Name:" fieldWidth:500 labelOnTop:true

	checkbox checkboxDoNotRender "Run only the Simulation and do not Render"

	checkbox checkboxSubmitSuspended "Submit the Job Suspended"
	
	checkbox checkboxRestoreSim "Restore the Simulation" across:2
	spinner spinnerRestoreFrame "Restore Frame" range:[-100000,100000,0] type:#integer fieldWidth:50 align:#left

	edittext editSimCacheOutput "Simulation Cache Output Directory Override:" fieldWidth:500 labelOnTop:true align:#left across:2
	button btnSimCacheOutput "Browse ..." align:#right offset:[0,16]
	on btnSimCacheOutput pressed do(
		simCacheOutputString = getSavePath caption:"Set Simulation Cache Output Location"
		if simCacheOutputString!=undefined then editSimCacheOutput.text = simCacheOutputString
	)

	edittext editRenderOutput "Render Output Directory Override:" fieldWidth:500 labelOnTop:true align:#left across:2
	button btnRenderOutput "Browse ..." align:#right offset:[0,16]
	on btnRenderOutput pressed do(
		renderOutputString = getSavePath caption:"Set Render Output Location"
		if renderOutputString!=undefined then editRenderOutput.text = renderOutputString
	)

	button runSim "Run" align:#center offset:[0,20] width:100 height:35
	on runSim pressed do(
		if managerText.text=="" then(
			messageBox "Please type in the Name or IP address of the Backburner manager before submitting the simulation!"
		)else(
			--If there are selected simulators, use only those
			PhoenixFDObjectsAll = for o in selection where classof o == LiquidSim or classof o == FireSmokeSim or classof o == PHXSimulator collect o
			if PhoenixFDObjectsAll.count == 0 do( --Otherwise, use all PhoenixFD simulators
				PhoenixFDObjectsAll = for o in objects where classof o == LiquidSim or classof o == FireSmokeSim or classof o == PHXSimulator collect o
			)
			-- Filter out the hidden particle group nodes from the actual simulators:
			PhoenixFDObjectsToSim = #()
			for i=1 to PhoenixFDObjectsAll.count do(
				local phxUserPropGroup = getUserProp PhoenixFDObjectsAll[i] "phxgroupname"
				local phxUserPropSysId = getUserProp PhoenixFDObjectsAll[i] "phx_system_id"
				if phxUserPropGroup==undefined and phxUserPropSysId==undefined do(
					append PhoenixFDObjectsToSim PhoenixFDObjectsAll[i].inode.handle
				)
			)

			-- Create a fake helper node to hold the list of the selected simulators, as we can't pass that info otherwise to the job:
			newHelperNode = point()
			newHelperNode.name = "PhoenixBackburnerHelper"
			setuserprop newHelperNode "PhoenixBackburnerNodeHandles" PhoenixFDObjectsToSim

			-- Save a pre-render command in the scene that would start the simulation when the job is executed:
			simulateCommand = "PhoenixFDObjectsToSim = execute(getuserprop $PhoenixBackburnerHelper \"PhoenixBackburnerNodeHandles\"); for phoenixNodeHandle in PhoenixFDObjectsToSim do ( phoenixNode = (maxOps.getNodeByHandle phoenixNodeHandle); try( "
			if checkboxRestoreSim.checked then(
				simulateCommand += "A_StartSim phoenixNode \"\" "
				simulateCommand += spinnerRestoreFrame.value as string
				simulateCommand += ";"
			)else(
				simulateCommand += "A_StartSim phoenixNode;"
			)
			simulateCommand += "A_Wait phoenixNode; )catch( print \"An exception occurred while simulating Phoenix FD!\" ) )"
			callbacks.addScript #preRender simulateCommand id:#PhoenixBackburnerScript persistent:true

			mymanager = NetRender.GetManager()
			mymanager.connect #manual (managerText.text)

			thejob = mymanager.newjob()
			thejob.suspended = checkboxSubmitSuspended.checked
			-- Generate a unique name for each job:
			timeArray = getLocalTime()
			theJob.name += "_"
			theJob.name += (formattedPrint timeArray[1] format:"04d")
			theJob.name += (formattedPrint timeArray[2] format:"02d")
			theJob.name += (formattedPrint timeArray[4] format:"02d")
			theJob.name += (formattedPrint timeArray[5] format:"02d")
			theJob.name += (formattedPrint timeArray[6] format:"02d")
			theJob.name += (formattedPrint timeArray[7] format:"02d")
			theJob.name += (formattedPrint timeArray[8] format:"03d")
			if checkboxDoNotRender.checked then(
				thejob.fromFrame = 0
				thejob.toFrame = 0
				thejob.outputWidth = 2
				thejob.outputHeight = 2
				theJob.frameOutputName = ""
			)else(
				thejob.fromFrame = animationRange.start
				thejob.toFrame = animationRange.end
				if editRenderOutput.text!="" then(
					renderPath = editRenderOutput.text
					if renderPath[renderPath.count]!="\\" then(
						renderPath += "\\"
					)
					makeDir renderPath all:true
					if theJob.frameOutputName!=undefined and theJob.frameOutputName!="" then(
						renderFileName = filenameFromPath theJob.frameOutputName
					)else if rendOutputFilename!=undefined and rendOutputFilename!="" then(
						renderFileName = filenameFromPath rendOutputFilename
					)else if maxfilename!=undefined and maxfilename!="" then(
						renderFileName = maxfilename + ".jpg"
					)else(
						renderFileName = "Untitled.jpg"
					)
					theJob.frameOutputName = renderPath + renderFileName
				)
			)

			-- Substitute the simulation paths if there is an override:
			backupSimOutputPaths = #()
			backupSimInputPaths = #()
			if editSimCacheOutput.text!="" then(
				for phoenixNodeHandle in PhoenixFDObjectsToSim do(
					phoenixNode = (maxOps.getNodeByHandle phoenixNodeHandle)
					append backupSimOutputPaths phoenixNode.simoutput
					append backupSimInputPaths phoenixNode.rendinput

					phoenixNode.rendinput = "$(same_as_output)"

					simPath = editSimCacheOutput.text
					if simPath[simPath.count]!="\\" then(
						simPath += "\\"
					)
					-- If the output path contains any Phoenix macros, we can't be certain about the filename:
					if (findString phoenixNode.simoutput "$")==undefined then(
						simFileName = filenameFromPath phoenixNode.simoutput
					)else(
						simFileName = "ND" + (phoenixNodeHandle as string) + "_" + phoenixNode.name + "_####.aur"
					)
					phoenixNode.simoutput = simPath + simFileName
				)
			)

			if groupText.text!="" then(
				srv_list = mymanager.getservers filter:#group key:(groupText.text)
				thejob.submit servers:srv_list
			)else(
				thejob.submit()
			)

			-- Reset the scene to the way it was before:
			if editSimCacheOutput.text!="" then(
				for i=1 to PhoenixFDObjectsToSim.count do(
					phoenixNode = (maxOps.getNodeByHandle PhoenixFDObjectsToSim[i])
					phoenixNode.simoutput[i] = backupSimOutputPaths[i]
					phoenixNode.rendinput[i] = backupSimInputPaths[i]
				)
			)
			delete newHelperNode
			callbacks.removeScripts id:#PhoenixBackburnerScript

			mymanager.disconnect()
		)
	)

	on BackburnerSim open do(
		phoenixFDiniFile = getDir #plugcfg + "\\phoenixfd.ini"

		backburnerManagerName = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerManager"
		managerText.text = backburnerManagerName

		backburnerGroupName = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerGroup"
		groupText.text = backburnerGroupName

		backburnerDoNotRender = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerDoNotRender"
		if backburnerDoNotRender=="false" then(
			checkboxDoNotRender.checked = false
		)else( -- if true or empty, set it to true by default
			checkboxDoNotRender.checked = true
		)

		backburnerSubmitSuspended = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerSubmitSuspended"
		if backburnerSubmitSuspended=="true" then(
			checkboxSubmitSuspended.checked = true
		)else( -- if false or empty, set it to false by default
			checkboxSubmitSuspended.checked = false
		)
		
		backburnerRestoreSim = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRestoreSim"
		if backburnerRestoreSim=="true" then(
			checkboxRestoreSim.checked = true
		)else( -- if false or empty, set it to false by default
			checkboxRestoreSim.checked = false
		)
		
		backburnerRestoreFrame = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRestoreFrame"
		if backburnerRestoreFrame!="" then(
			spinnerRestoreFrame.value = backburnerRestoreFrame as integer
		)else(
			spinnerRestoreFrame.value = 0
		)

		backburnerSimCacheOutPath = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerSimCacheOutPath"
		editSimCacheOutput.text = backburnerSimCacheOutPath

		backburnerRenderOutPath = getINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRenderOutPath"
		editRenderOutput.text = backburnerRenderOutPath
	)

	on BackburnerSim close do(
		phoenixFDiniFile = getDir #plugcfg + "\\phoenixfd.ini"

		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerManager" managerText.text
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerGroup" groupText.text
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerDoNotRender" (checkboxDoNotRender.checked as string)
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerSubmitSuspended" (checkboxSubmitSuspended.checked as string)
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRestoreSim" (checkboxRestoreSim.checked as string)
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRestoreFrame" (spinnerRestoreFrame.value as string)
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerSimCacheOutPath" editSimCacheOutput.text
		setINISetting phoenixFDiniFile "SIMSUBMIT" "BackburnerRenderOutPath" editRenderOutput.text
	)
)

macroScript PhoenixShowBackburnerDialog category:"PhoenixFD"
(
	destroyDialog BackburnerSim
	createDialog BackburnerSim 600 310
)

rollout DeadlineSim "Phoenix FD Deadline Simulation"
(
	edittext cmdRepoText "Deadline Repository:" fieldWidth:500 labelOnTop:true align:#left across:2
	button cmdRepoBtn "Browse ..." align:#right offset:[0,16]
	on cmdRepoBtn pressed do(
		cmdRepoString = getSavepath caption:"Set Deadline Repository location" filename:"" types:"MAXScript(*.ms)|*.ms"
		if cmdRepoString!=undefined then cmdRepoText.text = cmdRepoString
	)

	edittext scriptLocText "Simulation Script Location:" fieldWidth:500 labelOnTop:true align:#left across:2
	button scriptLocBtn "Browse ..." align:#right offset:[0,16]
	on scriptLocBtn pressed do(
		scriptLocString = getOpenFileName caption:"Set Simulation Script location" filename:"" types:"MAXScript(*.ms)|*.ms"
		if scriptLocString!=undefined then scriptLocText.text = scriptLocString
	)

	button runSim "Run" align:#center offset:[0,20] width:100 height:35
	on runSim pressed do(
		MAXFile = maxfilepath+maxfilename
		if cmdRepoText.text=="" or scriptLocText.text=="" then(
			messageBox "Please fill all the fields above before running the simulation."
		)else if (doesFileExist scriptLocText.text)==false then(
			messageBox "The Simulation Script cannot be found in the specified location!"
		)else if MAXFile=="" then(
			messageBox "Please save the scene before running the simulation!"
		)else(
			global SMTDSettings
			global SMTDFunctions
			global SMTDPaths

			--If there are selected simulators, use only those
			local PhoenixFDObjectsAll = for o in selection where classof o == LiquidSim or classof o == FireSmokeSim or classof o == PHXSimulator collect o
			if PhoenixFDObjectsAll.count == 0 do( --Otherwise, use all PhoenixFD simulators
				PhoenixFDObjectsAll = for o in objects where classof o == LiquidSim or classof o == FireSmokeSim or classof o == PHXSimulator collect o
			)
			-- Filter out the hidden particle group nodes from the actual simulators:
			local PhoenixFDNamesToSim = #()
			for i=1 to PhoenixFDObjectsAll.count do(
				local phxUserPropGroup = getUserProp PhoenixFDObjectsAll[i] "phxgroupname"
				local phxUserPropSysId = getUserProp PhoenixFDObjectsAll[i] "phx_system_id"
				if phxUserPropGroup==undefined and phxUserPropSysId==undefined do(
					append PhoenixFDNamesToSim PhoenixFDObjectsAll[i].name
				)
			)
			if PhoenixFDNamesToSim.count == 0 do(
				messageBox "There are no Phoenix simulators in the scene! Aborting."
				return "There are no Phoenix simulators in the scene! Aborting."
			)

			local theNetworkRoot = cmdRepoText.text
			local remoteScript = theNetworkRoot + @"\submission\3dsmax\main\SubmitMaxToDeadline_Functions.ms"
			local localScript = getDir #userscripts + "\\SubmitMaxToDeadline_Functions.ms"
			if (doesFileExist remoteScript)==false do(
				messageBox "The path to the Deadline repo is invalid! Aborting."
				return "The path to the Deadline repo is invalid! Aborting."
			)

			if SMTDFunctions == undefined do(
				deleteFile localScript
				copyFile remoteScript localScript
				fileIn localScript
			)

			SMTDFunctions.loadSettings()
			SMTDSettings.JobName = maxFileName + " [PhoenixFD Simulation]"
			SMTDSettings.Comment = "Simulating PhoenixFD"
			SMTDSettings.Priority = 50
			SMTDSettings.ChunkSize = 1
			SMTDSettings.SubmitAsMXSJob = true
			renderSceneDialog.close()
			rendTimeType = 3
			rendStart = 1
			rendEnd = PhoenixFDNamesToSim.count
			local MAXScriptFile = scriptLocText.text

			local maxFileToSubmit = SMTDPaths.tempdir + maxFileName
			SMTDFunctions.SaveMaxFileCopy maxFileToSubmit

			local SubmitInfoFile = SMTDPaths.tempdir + "\\max_submit_info.job"
			local JobInfoFile = SMTDPaths.tempdir+ "\\max_job_info.job"

			SMTDFunctions.CreateSubmitInfoFile SubmitInfoFile
			SMTDFunctions.CreateJobInfoFile JobInfoFile
			theHandle = openFile JobInfoFile mode:"at"
			with printAllElements on format "PhoenixFDToSimulate=%" PhoenixFDNamesToSim to:theHandle
			close theHandle

			local initialArgs = "\""+SubmitInfoFile+"\" \""+JobInfoFile+"\" \""+maxFileToSubmit+"\" \""+MAXScriptFile+"\" "
			local result = SMTDFunctions.waitForCommandToComplete initialArgs SMTDSettings.TimeoutSubmission

			local renderMsg = SMTDFunctions.getRenderMessage()
			SMTDFunctions.getJobIDFromMessage renderMsg

			if result == #success then(
				messageBox ("Submitted successfully as Job "+SMTDSettings.DeadlineSubmissionLastJobID+".\nPlease check the Deadline Monitor or the MAXScript listener for more details.")
				format "Submitted successfully as Job %.\n\n%\n\n"\
					SMTDSettings.DeadlineSubmissionLastJobID renderMsg
			)else(
				format "Job Submission FAILED.\n\n%" renderMsg
			)
		)
	)

	on DeadlineSim open do(
		phoenixFDiniFile = getDir #plugcfg + "\\phoenixfd.ini"

		deadlineRepoDir = getINISetting phoenixFDiniFile "SIMSUBMIT" "DeadlineSimRepoDir"
		if deadlineRepoDir!="" then(
			cmdRepoText.text = deadlineRepoDir
		)else(
			cmdRepoText.text = ""
		)

		deadlineSimScriptPath = getINISetting phoenixFDiniFile "SIMSUBMIT" "DeadlineSimScriptPath"
		if deadlineSimScriptPath!="" then(
			scriptLocText.text = deadlineSimScriptPath
		)else(
			scriptLocText.text = getDir #maxroot + "scripts\\PhoenixFD\\PhoenixFDSimulateDeadline.ms"
		)
	)

	on DeadlineSim close do(
		phoenixFDiniFile = getDir #plugcfg + "\\phoenixfd.ini"

		setINISetting phoenixFDiniFile "SIMSUBMIT" "DeadlineSimRepoDir" cmdRepoText.text
		setINISetting phoenixFDiniFile "SIMSUBMIT" "DeadlineSimScriptPath" scriptLocText.text
	)
)

macroScript PhoenixShowDeadlineDialog category:"PhoenixFD"
(
	destroyDialog DeadlineSim
	createDialog DeadlineSim 600 160
)

if menuMan.registerMenuContext 0x55ae91fd then
(
	theMainMenu = menuMan.getMainMenuBar()
	theMenu = menuMan.createMenu "PhoenixFD"
	theSubMenu = menuMan.createSubMenuItem "PhoenixFD" theMenu
	theMainMenu.addItem theSubMenu (theMainMenu.numItems()+1)

	theAction1 = menuMan.createActionItem "PhoenixShowBackburnerDialog" "PhoenixFD"
	theAction1.setTitle "Submit Simulation to Backburner..."
	theAction1.setUseCustomTitle true
	theMenu.addItem theAction1 (theMenu.numItems()+1)

	theAction2 = menuMan.createActionItem "PhoenixShowDeadlineDialog" "PhoenixFD"
	theAction2.setTitle "Submit Simulation to Deadline..."
	theAction2.setUseCustomTitle true
	theMenu.addItem theAction2 (theMenu.numItems()+1)

	menuMan.updateMenuBar()
)
